#include <stdio.h>
#include <termios.h>

static struct termios ttysave;
static struct termios ttycur;

int xread(int fd, void *buf, int nbytes) {
	int r;
	int ret = 0;
retry:
	r = read(fd,buf, nbytes);
	if (r == nbytes || r == 0) return ret + nbytes;
	if (r < 0) {
		if (ret == 0) return r;
		return ret;
	}
	nbytes -= r;
	ret += r;
	goto retry;
}

int fd;

unsigned char ibuf[1024];
unsigned char obuf[1024];

int clk_mode = 0;
int ms1 = 1;
int ms2 = 1;
int ck1 = 1;
int ck2 = 0;
int ckt = 0;

set_mreg() {
  int ret;
  obuf[0] = 0x1c;
  obuf[1] = 0x01;
  obuf[2] = (ckt << 4) | (ck2 << 3) | (ck1 << 2) | (ms2 << 1) | ms1 
            | (clk_mode << 5);
  write (fd, obuf, 3);
  ret = read (fd, ibuf, 3);
  if (ret != 3) printf("set_mreg error\n");
}

set_ms2(int data) {
  if (data) ms2 = 1; else ms2 = 0;
  set_mreg();
}

main() {
	int i,ret;
	int cnt = 0;
	int err_cnt = 0;
	speed_t si,so;

	fd = open("/dev/ttyUSB0",2);
	if (fd < 0) {
		perror("open tty");
		exit(1);
	}
	tcgetattr(fd, &ttysave);
        ttysave.c_iflag |= INLCR | ICRNL;
        ttysave.c_lflag |= ICANON | ECHO;
        ttysave.c_oflag |= OPOST | ONLCR;
        ttycur = ttysave;
        ttycur.c_iflag = IGNPAR | IGNBRK;
        ttycur.c_oflag = ONLCR;
        ttycur.c_cflag = CREAD | CS8;
        ttycur.c_lflag &= ~(ICANON | IEXTEN | ECHO | ISIG);
        ttycur.c_cc[VTIME] = 0;
        ttycur.c_cc[VMIN] = 1;
	cfsetispeed(&ttycur, B4000000);
	cfsetospeed(&ttycur, B4000000);
        tcsetattr(fd, TCSANOW, &ttycur);
#if 0
	si = cfgetispeed(&ttycur);
	printf("ispeed = %o \n",si);
	so = cfgetospeed(&ttycur);
	printf("ospeed = %o \n",so);
	exit(1);	

#endif
        ckt = 0;
        ck1 = 0;
        ck2 = 0;
        ms2 = 1;
        ms1 = 1;
        set_mreg();
	printf("getchar\n");
	getchar();
        ck1 = 1;
        ck2 = 1;
        ms2 = 0;
        ms1 = 0;
        set_mreg();
	printf("getchar\n");
	getchar();

        ckt = 0;
        ck1 = 1;
        ck2 = 0;
        ms1 = 1;
        ms2 = 0;
        set_mreg();

	obuf[0] = 0x00 | 0x7;
	obuf[1] = 0x04;
	obuf[2] = 0xaa;
	obuf[3] = 0xaa;
	obuf[4] = 0xaa;
	obuf[5] = 0xaa;
	write (fd, obuf, 6);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]);

	obuf[0] = 0x00 | 0x7;
	obuf[1] = 0x04;
	obuf[2] = 0xaa;
	obuf[3] = 0xaa;
	obuf[4] = 0xaa;
	obuf[5] = 0xaa;
	write (fd, obuf, 6);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]);
	printf("getchar\n");
	getchar();


        ckt = 0;
        ck2 = 0;
	set_ms2(0);
	set_ms2(1);
	set_ms2(0);
        sleep(1);

/* programming enable*/
	printf("programming enable \n");
        /* 0011 0000 ,0000 0000, 0000 0000 , 0x00 */ 
	obuf[0] = 0x08; /* chan = 1 */
	obuf[1] = 0x04+3;
	obuf[2] = 0xac;
	obuf[3] = 0x53;
	obuf[4] = 0x00;
	obuf[5] = 0x00;
	obuf[6] = 0xff;
	obuf[7] = 0xff;
	obuf[8] = 0xff;
	write (fd, obuf, 6+3);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]
,ibuf[6],ibuf[7],ibuf[8]);
	printf("getchar\n");
/* signature byte 1E 90 05 page 45 */
	printf("read signature bytes 0\n");
        /* 0011 0000 ,0000 0000, 0000 0000 , 0x00 */ 
	obuf[0] = 0x08; /* chan = 1 */
	obuf[1] = 0x04;
	obuf[2] = 0xac;
	obuf[3] = 0x53;
	obuf[4] = 0x00;
	obuf[5] = 0x00;
	write (fd, obuf, 6);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]);
	printf("getchar\n");
        getchar();

#if 0
/* signature byte 1E 90 05 page 45 */
	printf("read fuse bits \n");
        /* 0011 0000 ,0000 0000, 0000 0000 , 0x00 */ 
	obuf[0] = 0x08; /* chan = 1 */
	obuf[1] = 0x04+3;
	obuf[2] = 0x50;
	obuf[3] = 0x00;
	obuf[4] = 0x00;
	obuf[5] = 0x00;
	obuf[6] = 0xff;
	obuf[7] = 0xff;
	obuf[8] = 0xff;
	write (fd, obuf, 6+3);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]
,ibuf[6],ibuf[7],ibuf[8]);
	printf("getchar\n");
        getchar();

#if 0
	set_ms2(1);
	set_ms2(0);

	printf("read program memory at 0x100\n");
        /* 0010 0000 ,0000 0001, 0000 0000 , 0x12 */ 

	obuf[0] = 0x08; /* chan = 1 */
	obuf[1] = 0x04;
	obuf[2] = 0x20;
	obuf[3] = 0x01;
	obuf[4] = 0x00;
	obuf[5] = 0x51;
	write (fd, obuf, 6);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]);
	printf("getchar\n");
	getchar();


	set_ms2(1);
	set_ms2(0);

	printf("write program memory at 0x100 to a5\n");
        /* 0100 0000 ,0000 0001, 0000 0000 , 0xa5 */ 

	obuf[0] = 0x08; /* chan = 1 */
	obuf[1] = 0x04;
	obuf[2] = 0x40;
	obuf[3] = 0x01;
	obuf[4] = 0x00;
	obuf[5] = 0xa5;
	write (fd, obuf, 6);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]);
	printf("getchar\n");

	set_ms2(1);
	set_ms2(0);

	printf("read program memory at 0x100\n");
        /* 0010 0000 ,0000 0001, 0000 0000 , 0x12 */ 

	obuf[0] = 0x08; /* chan = 1 */
	obuf[1] = 0x04;
	obuf[2] = 0x20;
	obuf[3] = 0x01;
	obuf[4] = 0x00;
	obuf[5] = 0x51;
	write (fd, obuf, 6);
	ret = read (fd, ibuf, 32);
	printf(" read ret = %d, ibuf = %02x %02x %02x %02x %02x %02x\n",ret,ibuf[0],ibuf[1],ibuf[2],ibuf[3],ibuf[4],ibuf[5]);
	printf("getchar\n");
#endif
#endif
}
