-- usb_sio.vhd (version 009)
--    desgined by suz
--
-- cmd format 
--
--     7   6   5   4   3   2   1   0  
--   +---+---+---+-------+-----------+
--   | 0 | 0 | 0 | chan  |  bits     |
--   +---+---+---+-------+-----------+
-- chan : 0 => external 1
--        1 => external 2
--        3 => internal reg
--        internal reg
--            7    6    5    4    3    2    1    0
--          +----+----+----+----+----+----+----+----+
--          |  X   X    X  |CKT |CK2 |CK1 |MS2 |MS1 |
--          +----+----+----+----+----+----+----+----+
--
--          CKT : ck1,ck2 timing
--
--                  |  DATAOUT   |        |  DATAOUT   |
--                        @                     @
--                        DATAIN                DATAIN
--          CKT = 0,CKX = 0              CKT = 0,CKX = 1
--                  ______|~~~~~~|        ~~~~~~|_______|
--          CKT = 1,CKX = 0              CKT = 1,CKX = 1
--                  |~~~~~|_______       |______|~~~~~~~
--
-- bits : last byte's shift count
---       1-7 => 1-7 bits 
--        0   =>   8 bits
-- bytes format
--     7   6   5   4   3   2   1   0  
--   +---+---+---+-------+-----------+
--   | 0 |   bytes                   | 
--   +---+---+---+-------+-----------+
-- bytes => 1 .. 127
--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity usb_sio is
    Port ( DATA    : inout std_logic_vector(7 downto 0) bus;
           CLK     : in std_logic;
           RXF     : in std_logic;
           TXF     : in std_logic;
           RESET   : in std_logic;
           RD      : out std_logic;
           WR      : out std_logic;
           EEREQ   : out std_logic;
           --DCNT    : out std_logic_vector(3 downto 0);
           ODIR     : out std_logic_vector(1 downto 0);
           OSTATE   : out std_logic_vector(1 downto 0);
           OCLK     : out std_logic;

           DI1     : in std_logic;
           DO1     : inout std_logic;
           CK1     : out std_logic;
           MS1     : out std_logic;
           DI2     : in std_logic;
           DO2     : inout std_logic;
           CK2     : out std_logic;
           MS2     : out std_logic

                          );
end usb_sio;

--------------------------------------------------------------------------

architecture behavioral of usb_sio is

        signal state: std_logic_vector(1 downto 0);
            -- "00" before-read
            -- "01" read
            -- "10" befor-write
            -- "11" write

        signal dir: std_logic_vector(1 downto 0);
            -- "00" NONE;
            -- "01" CMD
            -- "10" BYTES
            -- "11" LATCH
        signal cmd   : std_logic_vector(7 downto 0);
        signal bytes : std_logic_vector(7 downto 0);
        signal latch : std_logic_vector(7 downto 0);
        signal lrxf : std_logic := '1';
        signal ltxf : std_logic := '0';
        signal bits : std_logic_vector(3 downto 0);
        signal do_clk_a : std_logic := '0';
        signal do_clk_b : std_logic;
        signal do_clk : std_logic;
        signal do_shift : std_logic := '0';
        signal do_load : std_logic := '0';
        signal stat2 : std_logic_vector(1 downto 0) := "00";
        signal m_reg : std_logic_vector(5 downto 0) := "000000";
        signal iclk : std_logic ;--:= '0';
        --signal icnt : std_logic_vector(0 downto 0) := "0";

begin
    MS1 <= '1' when (m_reg(0) = '1') else '0';
    MS2 <= '1' when (m_reg(1) = '1') else '0';
    EEREQ <= 'Z';

    ODIR(1) <= dir(1);
    ODIR(0) <= dir(0);
    OSTATE(1) <= state(1);
    OSTATE(0) <= state(0);
    OCLK <= iclk; 

    do_clk_b <= do_shift and (not do_load) ;
    do_clk <= (do_clk_a and do_clk_b and (not iclk)) when (m_reg(4) = '1') else
              (do_clk_a and iclk);

    RD <= '0' when (stat2 = "00" and state = "01") else '1'; 
    WR <= '1' when (stat2 = "10" and state = "11") else '0'; 

    CK1 <= (not m_reg(2)) when (do_clk = '1' and cmd(4 downto 3) = "00") else
            m_reg(2);
    CK2 <= (not m_reg(3)) when (do_clk = '1' and cmd(4 downto 3) = "01") else
            m_reg(3);

--process (CLK) begin
--    if (CLK' event) then
--        icnt <= icnt - 1;
--        if (icnt = "0") then
--             iclk <= not iclk;
--             icnt <= m_reg(5 downto 5);
--        end if;
--    end if;
--end process;
  iclk <= CLK;
process (iclk) begin
    if (iclk' event) then
	 end if;
    if (iclk' event and iclk = '0') then
        if (RESET = '0') then
        else
            DATA <= (others => 'Z');
            DO1 <= 'Z';
            DO2 <= 'Z';
            do_clk_a <= '0';
            case state is
                when "00" =>
                    if (lrxf = '0') then
                        state <= "01";
                        case dir is
                            when "01" => dir <= "10";
                            when "10" => dir <= "11";
                            when "11" => 
                                if (bytes(6 downto 0) = 0) then
                                    dir <= "01";
                                else
                                    dir <= "11";
                                end if;
                            when others => dir <= "01";
                        end case;
                    end if;
                when "01" =>
                    state <= "10";
                when "10" =>
                    if (cmd(7) = '1' 
                        or ((dir(1) = '1') and (bytes(7) = '1'))) then
                        dir <= "00";
                        state <= "00";
                    elsif (do_shift = '1') then
                        -- DO1 <= latch(7); -- for test
                        do_clk_a <= '1';
                        case cmd(4 downto 3) is
                             when "00" => DO1 <= latch(7);
                             when "01" => DO2 <= latch(7);
                             when others => null;
                        end case;
                    elsif (ltxf = '0') then
                        state <= "11";
                        case dir is
                            when "01" => DATA <= cmd;
                            when "10" => DATA <= bytes;
                            when "11" => DATA <= latch;
                            when others => DATA <= (others => 'Z');
                        end case;
                    end if;
                when others =>
                    state <= "00";
            end case;
        end if;
    end if;
    if (iclk' event and iclk = '1') then
        if (RESET = '0') then
            do_shift <= '0';
            do_load <= '0';
        else
            lrxf <= RXF;
            ltxf <= TXF;
            stat2 <= state;

            if (state = "01") then
                do_shift <= '0';
                do_load <= '0';
                case dir is
                    when "01" => cmd <= DATA;
                    when "10" => bytes <= DATA;
                    when "11" => latch <= DATA;
                       if bytes(6 downto 0) = "000001" 
                         and not (cmd(2 downto 0) = "000")
                       then
                            bits <= '0' & cmd(2 downto 0);
                       else
                            bits <= "1000";
                       end if;
                       do_shift <= '1';
                       do_load <= '1';
                       bytes(6 downto 0) <= bytes(6 downto 0) - 1;
                    when others => null;
                end case;
            end if;
            if (state = "10" and do_shift = '1') then
                 do_load <= '0';
                 if (bits = "0000" ) then
                     do_shift <= '0';
                     if (cmd(4 downto 3) = "11") then
                         m_reg <= latch(5 downto 0);
                     end if;
							latch(0) <= DI1;
							latch(1) <= DI2;
							--latch(2) <= DO1;
							--latch(3) <= DO2;
                 end if;
                 bits <= bits - 1;
                 if (do_load = '0') then
                 case cmd(4 downto 3) is
                     when "00" => latch <= latch(6 downto 0) & DI1;
                     when "01" => latch <= latch(6 downto 0) & DI2;
                     when others => null;
                 end case;
                 end if;
            end if;
        end if;
    end if;
end process;


end behavioral;
